{-
/***************************************************************************
 *   Copyright (C) 2004 by Peter Robinson,listener@thaldyron.com           *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         * 
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
-}

module Main where

import Prelude hiding (catch)
import IO hiding (catch)
import System.Posix.Signals
import System.Exit
import Control.Monad
import Control.Exception
import Control.Concurrent
import System.Environment
import System.Cmd

import CmdOpts
import InputParser
import Definitions

----------------------------------------------------------------------------------------------------

commonOpts      = ["-M " ++ memSizeForIndex,         -- add extra memory to speedup indexing
                   "-B"                              -- use a bigger hashtable to speedup indexing
                  ]
----------------------------------------------------------------------------------------------------

main :: IO () 
main = catch (
         do argStr       <- getArgs
--          (flags,args) <- cmdOpts argStr          -- not used at the moment since almost everything
                                                    -- can be customized in Definitions.hs
            printDebug "fsysmond daemon running..."
            printDebug (concatMap (' ':) (glimpseIndexCmd:(commonOpts++[indexRootDir])))
            system (concatMap (' ':) (glimpseIndexCmd:(commonOpts++[indexRootDir])))
            devHandle    <- openFile device ReadMode
            printDebug ("listening on " ++ device)
            catchJust asyncExceptions 
                      (forever (mainLoop devHandle)) 
                      (cleanup devHandle)
            return ()
       ) errorHandler
       ---------------------------------------------------------------------------------------------     
       where
       cleanup :: Handle -> AsyncException -> IO ()
       cleanup devHandle ex  = hClose devHandle


----------------------------------------------------------------------------------------------------

mainLoop :: Handle    -- file handle of the device
         -> IO ()
mainLoop dev = do inp <- hGetLine dev
                  printDebug ("Reading Input from " ++ device ++ "\n" ++ inp)
                  threadDelay daemonDelay -- unfortunately this has to be done to make sure that the
                                          -- kernel is already finished with its work when the index
                                          -- command is run => race condition                 
                  (runGlimpseIndex . createIndexCmd . parseInput) inp
                  return ()      
                  ----------------------------------------------------------------------------------     
                  where
                  createIndexCmd :: ParseRes     -- result of the previous parse
                                 -> Maybe String -- returns either a command string or Nothing
                  createIndexCmd (AddFile fname) 
                          = Just (concatMap (' ':) ([glimpseIndexCmd] ++ commonOpts ++ ["-a",fname]))
                          
                  createIndexCmd (RemFile fname) 
                          = Just (concatMap (' ':) ([glimpseIndexCmd] ++ commonOpts ++ ["-f",indexRootDir]))
                  
                  createIndexCmd (UpdFile fname)
                          = Just (concatMap (' ':) ([glimpseIndexCmd] ++ commonOpts ++ ["-f",fname]))
                  
                  createIndexCmd Skip            = Nothing                                  
                  ----------------------------------------------------------------------------------
                  runGlimpseIndex :: Maybe String -> IO ()
                  runGlimpseIndex Nothing    = return ()
                  runGlimpseIndex (Just cmd) = do printDebug cmd
                                                  res <- system cmd -- TODO: parser command result (?)
                                                  return ()
                  

----------------------------------------------------------------------------------------------------
-- simulate the while(1) loop of C/C++
forever :: Monad m => m () -> m ()
forever a  = a >> forever a

----------------------------------------------------------------------------------------------------

-- print out an error message          
errorHandler :: Exception
             -> IO ()

errorHandler ex = print ex   

----------------------------------------------------------------------------------------------------

printDebug :: String -- string to be printed in debug mode
           -> IO ()
printDebug str
  | debug     = putStrLn str
  | otherwise = return ()
  